/*
 * File:   tetris_pic16f1827.c
 * Author: kikusan
 *
 * Created on 2012/12/13, 14:12
 */

/*
 * $B%F%H%j%9(B
 * $BEE8;%*%s$G$7$P$i$/$9$k$H%G%b%2!<%`$rI=<(!#2?$+%9%$%C%A$r2!$9$H%2!<%`%9%?!<%H$7$^$9!#(B
 * SW1 $B%V%m%C%/$N1&0\F0(B
 * SW2 $B%V%m%C%/$N:80\F0(B
 * SW3 $BL$;HMQ(B
 * SW4 $B%V%m%C%/$N2sE>(B
 */

// PIC16F1827 Configuration Bit Settings
#pragma config FOSC = INTOSC, WDTE = OFF, PWRTE = ON,MCLRE = OFF, CP = OFF
#pragma config CPD = OFF, BOREN = ON, CLKOUTEN = OFF, IESO = ON,FCMEN = ON
#pragma config WRT = OFF,PLLEN = ON, STVREN = ON, BORV = LO, LVP = OFF

/******************************************************************************
 $B%$%s%/%k!<%I%U%!%$%k(B
******************************************************************************/
#include <xc.h>

/******************************************************************************
 $BDj?t$NDj5A!JD>@\;XDj!K(B
******************************************************************************/
#define OK      0   //$BLa$jCM(B
#define NG      -1  //$BLa$jCM(B
#define TRUE    0   //$BLa$jCM(B
#define FALSE   -1  //$BLa$jCM(B

//$B%=%U%H%&%'%"%?%$%^(B ----------------------------------------------------------
#define T0      1000    //T0(1000ms)    $BBT5!Cf$NI=<(;~4V(B
#define T1      3000    //T1(3000ms)    $B%G%b2hLL$K0\9T$9$k$^$G$N;~4V(B
#define T2      3000    //T2(3000ms)    $B%2!<%`%*!<%P!<$NE@?tI=<(;~4V(B
#define T3      500     //T3(500ms)     $B%G%b2hLL$N5<;w%9%$%C%AF~NO4V3V(B

// $B%F%H%j%9(B -------------------------------------------------------------------
#define FIELD_WIDTH     8       //$B%2!<%`%U%#!<%k%I(B($B2#(B)
#define FIELD_HEIGHT    8       //               ($B=D(B)
#define PIECE_LEFT      2       //$B%V%m%C%/:80\F0(B
#define PIECE_RIGHT     4       //$B%V%m%C%/1&0\F0(B
#define PIECE_DOWN      8       //$B%V%m%C%/2<0\F0(B
#define MOVE_TIM_1      500     //$B%V%m%C%/0\F04V3V(B(500$B!_(B2ms=1.00s)
#define MOVE_TIM_2      350     //$B%V%m%C%/0\F04V3V(B(350$B!_(B2ms=0.70s)
#define MOVE_TIM_3      150     //$B%V%m%C%/0\F04V3V(B(150$B!_(B2ms=0.30s)

/******************************************************************************
 $BDj?t%(%j%"$NDj5A(B(ROM)
******************************************************************************/
//$B%9%-%c%s%G!<%?(B
const unsigned char ScanData[8] = {0x01,0x02,0x04,0x08,0x10,0x20,0x40,0x80};

//$B%-%c%i%/%?%G!<%?(B(4$B!_(B8)
const unsigned char LEDDispData[][4] = {
    {0x00,0xff,0x81,0xff},  // 0
    {0x00,0x02,0xff,0x00},  // 1
    {0x00,0xf1,0x91,0x9f},  // 2
    {0x00,0x89,0x89,0xff},  // 3
    {0x00,0x1f,0x10,0xff},  // 4
    {0x00,0x8f,0x89,0xf9},  // 5
    {0x00,0xff,0x89,0xf9},  // 6
    {0x00,0x0f,0x01,0xff},  // 7
    {0x00,0xff,0x89,0xff},  // 8
    {0x00,0x9f,0x91,0xff},  // 9
};

/******************************************************************************
 $B%0%m!<%P%kJQ?t$NDj5A$H%$%K%7%c%i%$%:(B(RAM)
******************************************************************************/
// $B%F%H%j%9$K4X$9$kJQ?t(B -------------------------------------------------------
int             GameStage = 0;      //$B%2!<%`%9%F!<%8(B
                                    // 0:$BBT5!Cf(B
                                    // 1:$B%2!<%`Cf(B
                                    // 2:$B%2!<%`%*!<%P!<(B
                                    //-1:$B%G%b2hLL(B
int             Point;              //$BE@?t(B
char            field[FIELD_WIDTH][FIELD_HEIGHT];	//$B%2!<%`%U%#!<%k%I(B
char            currentPiece[3][3]; //$B8=:_0\F0Cf$N%V%m%C%/(B
int             pieceLocation_x;    //$B%V%m%C%/$N0LCV(B(X$B:BI8(B)
int             pieceLocation_y;    //$B%V%m%C%/$N0LCV(B(Y$B:BI8(B)
unsigned int    MoveTimCount;       //$B%V%m%C%/0\F04V3V%+%&%s%?(B
unsigned char   BlockMoveFlag = 0;  //$B%V%m%C%/$r2<$KMn$H$9(B(=1)

// LED$BI=<($K4X78$7$?JQ?t(B ------------------------------------------------------
unsigned char   ScanCnt = 0;    //$B%9%-%c%s%+%&%s%?(B
unsigned char   DispBuf[8] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}; //$BI=<(%P%C%U%!(B

// $B%9%$%C%AF~NO$K4X78$7$?JQ?t(B -------------------------------------------------
unsigned char   SwData1  = 0;   //$B%U%!!<%9%H%j!<%I(B
unsigned char   SwData2  = 0;   //$B%@%V%k%j!<%I$K$h$j7hDj$7$?%G!<%?(B
unsigned char   SwData3  = 0;   //$BA02s$N%@%V%k%j!<%I$G7hDj$7$?%G!<%?(B
unsigned char   SwData4  = 0;   //0$B"*(B1$B$KJQ2=$7$?%G!<%?(B
unsigned char   SwStatus = 0;   //$B%9%$%C%AF~NO%9%F!<%?%9(B(0:$B%U%!!<%9%H%j!<%I(B/1:$B%@%V%k%j!<%I(B)

// $B%=%U%H%&%'%"%?%$%^$K4X78$7$?JQ?t(B -------------------------------------------
struct SoftTimer{           //$B%=%U%H%&%'%"%?%$%^$N9=B$BN%?%0(B
    unsigned char   Status; //$B%?%$%^%9%F!<%?%9(B
                            // 0:$BDd;_Cf(B($B%?%$%^L$;HMQ(B)
                            // 1:$B%9%?!<%H;XNa(B
                            // 2:$B%+%&%s%HCf(B
                            // 3:$B%+%&%s%H=*N;(B
    unsigned int    Count;  //$B%?%$%^%+%&%s%?(B
};
struct SoftTimer TimT0;     //T0$B%?%$%^(B
struct SoftTimer TimT1;     //T1$B%?%$%^(B
struct SoftTimer TimT2;     //T2$B%?%$%^(B
struct SoftTimer TimT3;     //T3$B%?%$%^(B

// $BMp?t(B -----------------------------------------------------------------------
unsigned char   ransuu0_9 = 0;  //0-9$B$NMp?t(B
unsigned char   ransuu0_6 = 0;  //0-6$B$NMp?t(B

/******************************************************************************
 $B4X?t$NDj5A(B
******************************************************************************/
void main(void);
void interrupt intr(void);
void intr_tmr0(void);
void dec_soft_timer(struct SoftTimer *,unsigned int);
void init_soft_timer(void);
void GameStandby(void);
void Game(void);
void GameOver(void);
void GameDemo(void);
int GetPieceTop(void);
int GetPieceBottom(void);
int GetPieceLeft(void);
int GetPieceRight(void);
void PieceToField(void);
int MovePiece(int);
int TurnPiece(void);
int AdjustLine(void);
void NextPiece(void);
void TimerProc(void);
void Paint(void);
void init_tetris(void);
void init_work_area(void);

/******************************************************************************
 $B3d$j9~$_(B(4$BHVCO%9%?!<%H(B)
******************************************************************************/
void interrupt intr(void)
{
    if (T0IF==1){intr_tmr0();}
}

/******************************************************************************
 TMR0 $B3d$j9~$_(B(2.048ms)
******************************************************************************/
void intr_tmr0(void)
{
    unsigned char d,s,i;
    int line, top;

    T0IF = 0;	// TMR0 Interrupt Clear

    // LED Display Scan -------------------------------------------------------
    s = ScanData[ScanCnt];
    d = DispBuf[ScanCnt];
    for(i=0; i<8; i++){
        if((s & 0x80)==0x80)    {RA2 = 1;}
        else                    {RA2 = 0;}
        if((d & 0x80)==0x80)    {RA1 = 0;}
        else                    {RA1 = 1;}
        RA0 = 1; RA0 = 0;
        s = s << 1;
        d = d << 1;
    }
    ScanCnt++; if(ScanCnt>=8){ScanCnt = 0;}

    // Switch Input -----------------------------------------------------------
    switch(SwStatus){
        case 0:
            SwData1 = ~PORTA & 0xf0;
            if (SwData1!=0) {SwStatus = 1;}
            else            {SwData2 = SwData3 =0;}
            break;
        case 1:
            if (SwData1==(~PORTA & 0xf0)){
                SwData2 = SwData1;
                SwData4 = SwData4 | (SwData2 & (~SwData3));
                SwData3 = SwData2;
            }
            SwStatus = 0;
            break;
    }

    //$B%V%m%C%/0\F0(B ------------------------------------------------------------
    if (GameStage==1 || GameStage==-1){
        MoveTimCount--;
        if (MoveTimCount==0){
            if      (Point<50)  {MoveTimCount = MOVE_TIM_1;}    //$BDcB.(B
            else if (Point<80)  {MoveTimCount = MOVE_TIM_2;}    //$BCfB.(B
            else                {MoveTimCount = MOVE_TIM_3;}    //$B9bB.(B
            BlockMoveFlag = 1;
        }
    }
    else{
        BlockMoveFlag = 0;
    }

    //$B%=%U%H%&%'%"%?%$%^(B ------------------------------------------------------
    if (TimT0.Status==1 || TimT0.Status==2){dec_soft_timer(&TimT0,T0/2);}   //T0
    if (TimT1.Status==1 || TimT1.Status==2){dec_soft_timer(&TimT1,T1/2);}   //T1
    if (TimT2.Status==1 || TimT2.Status==2){dec_soft_timer(&TimT2,T2/2);}   //T2
    if (TimT3.Status==1 || TimT3.Status==2){dec_soft_timer(&TimT3,T3/2);}   //T3
}

/******************************************************************************
 $B%=%U%H%&%'%"%?%$%^$N%G%/%j%a%s%H(B
-------------------------------------------------------------------------------
    $B0z?t(B    *pst    $B%=%U%H%&%'%"%?%$%^9=B$BN$N%]%$%s%?(B
            initial $B%?%$%^%+%&%s%?$N=i4|CM(B
******************************************************************************/
void dec_soft_timer(struct SoftTimer *pst,unsigned int initial)
{
    if (pst->Status==1){            //$B%?%$%^%9%?!<%H;XNa(B
        pst->Status =   2;          //$B%+%&%s%HCf%;%C%H(B
        pst->Count  =   initial;    //$B%?%$%^%+%&%s%?=i4|2=(B
    }
    pst->Count--;   //$B%+%&%s%?(B-1
    if (pst->Count==0){             //$B%+%&%s%?$,(B0$B$K$J$C$?(B
        pst->Status =   3;          //$B%+%&%s%H=*N;%;%C%H(B
    }
}

/******************************************************************************
 $B%=%U%H%&%'%"%?%$%^$N%$%K%7%c%i%$%:(B
******************************************************************************/
void init_soft_timer(void)
{
    TimT0.Status = 0; TimT1.Status = 0; TimT2.Status = 0; TimT3.Status = 0;
}

/******************************************************************************
 $B%a%$%s%W%m%0%i%`(B
******************************************************************************/
void main(void)
{
    // Option Registor Initialze
    OPTION_REG = 0x85;  // Prescaler = 64

    // Oscillator Initialize
    OSCCON = 0xf0;  // 4xPLL-Enable,8MHz = 32MHz

    // I/O Port Initialize
    ANSELA = 0x00;
    TRISA = 0xf0;
    PORTA = 0x0a;
    RA0 = 1; RA0 = 0;
    RA0 = 1; RA0 = 0;
    RA0 = 1; RA0 = 0;
    RA0 = 1; RA0 = 0;
    RA0 = 1; RA0 = 0;
    RA0 = 1; RA0 = 0;
    RA0 = 1; RA0 = 0;
    RA0 = 1; RA0 = 0;
    ANSELB = 0x00;
    TRISB = 0xff;

    // TMR0 Initialize
    TMR0 = 0x00;        // 32MHz/4/64/256 -> 2.048ms

    // Interrupt Initialize
    T0IE = 1;		// TMR0 Interrupt Enable
    GIE = 1;		// Interrupt Enable

    // Work Area Initialize
    init_tetris();

    // Main Loop --------------------------------------------------------------
    while(1){
        switch(GameStage){
            // $BBT5!Cf(B ---------------------------------------------------------
            case 0:
                GameStandby();
                break;

            // $B%2!<%`Cf(B -------------------------------------------------------
            case 1:
                Game();
                break;

            // $B%2!<%`%*!<%P!<(B -------------------------------------------------
            case 2:
                GameOver();
                break;

            // $B%G%b2hLL(B -------------------------------------------------------
            case -1:
                GameDemo();
                break;
        }

        // $BMp?t$N:n@.(B
        ransuu0_9 = ransuu0_9 + 1; if(ransuu0_9>9){ransuu0_9 = 0;}
        ransuu0_6 = ransuu0_6 + 1; if(ransuu0_6>6){ransuu0_6 = 0;}
    }
}

/******************************************************************************
 $B%2!<%`%9%?%s%P%$(B
******************************************************************************/
void GameStandby(void)
{
    if      (TimT1.Status==0){  //$B%G%b2hLL$X$N0\9T;~4V(B
        TimT1.Status = 1;
        DispBuf[0] = 0x00;
        DispBuf[1] = 0xc0;
        DispBuf[2] = 0xc2;
        DispBuf[3] = 0xe7;
        DispBuf[4] = 0xef;
        DispBuf[5] = 0xe7;
        DispBuf[6] = 0xc2;
        DispBuf[7] = 0xc0;
    }
    else if (TimT1.Status==3){  //$B%?%$%^=*N;(B
        TimT1.Status = 0;
        init_tetris();
        NextPiece();
        GameStage = -1;
    }

    if ((SwData4 & 0xf0)!=0){   //$B2?$+%9%$%C%A$,2!$5$l$?$i%2!<%`%9%?!<%H(B
        TimT1.Status = 0;
        init_tetris();
        NextPiece();
        GameStage = 1;
        SwData4 = 0;
    }
}

/******************************************************************************
 $B%2!<%`(B
******************************************************************************/
void Game(void)
{
    if      ((SwData4 & 0x10)==0x10){   //SW1$B$,2!$5$l$?(B
        MovePiece(PIECE_RIGHT);
        SwData4 = 0;
    }
    else if ((SwData4 & 0x20)==0x20){   //SW2$B$,2!$5$l$?(B
        MovePiece(PIECE_LEFT);
        SwData4 = 0;
    }
    else if ((SwData4 & 0x80)==0x80){   //SW4$B$,2!$5$l$?(B
        TurnPiece();
        SwData4 = 0;
    }

    if(BlockMoveFlag==1){   //$B%V%m%C%/$r2<$KMn$H$9(B
        BlockMoveFlag = 0;
        TimerProc();
    }

    Paint();    //$B%2!<%`%U%#!<%k%I$NI=<((B
}

/******************************************************************************
 $B%2!<%`%*!<%P!<(B
******************************************************************************/
void GameOver(void)
{
    int p10,p1;

    if      (TimT2.Status==0){
        TimT2.Status = 1;
        p10 = (Point / 10) % 10;
        p1  = Point % 10;
        DispBuf[0] = LEDDispData[p10][0];
        DispBuf[1] = LEDDispData[p10][1];
        DispBuf[2] = LEDDispData[p10][2];
        DispBuf[3] = LEDDispData[p10][3];
        DispBuf[4] = LEDDispData[ p1][0];
        DispBuf[5] = LEDDispData[ p1][1];
        DispBuf[6] = LEDDispData[ p1][2];
        DispBuf[7] = LEDDispData[ p1][3];
    }
    else if (TimT2.Status==3){
        TimT2.Status = 0;
        DispBuf[0] = DispBuf[1] = DispBuf[2] = DispBuf[3] = 0x00;
        DispBuf[4] = DispBuf[5] = DispBuf[6] = DispBuf[7] = 0x00;
        SwData4 = 0;
        GameStage = 0;
    }
}

/******************************************************************************
 $B%G%b%2!<%`(B
******************************************************************************/
void GameDemo(void)
{
    if      (TimT3.Status==0){TimT3.Status = 1;}
    else if (TimT3.Status==3){
        TimT3.Status = 1;
        if (pieceLocation_y>=0){
            switch(ransuu0_9){
                case 0:
                    MovePiece(PIECE_RIGHT); //$B1&0\F0(B
                    break;
                case 1:
                    MovePiece(PIECE_LEFT);  //$B:80\F0(B
                    break;
                case 2:
                    TurnPiece();            //$B2sE>(B
                    break;
                default:
                    break;                  //$B2?$b$7$J$$(B
            }
        }
    }

    if(BlockMoveFlag==1){   //$B%V%m%C%/$r2<$KMn$H$9(B
        BlockMoveFlag = 0;
        TimerProc();
    }

    Paint();

    if ((SwData4 & 0xf0)!=0){   //$B2?$+%9%$%C%A$,2!$5$l$?$i%2!<%`%9%?!<%H(B
        TimT3.Status = 0;
        init_tetris();
        NextPiece();
        GameStage = 1;
        SwData4 = 0;
    }
}

/******************************************************************************
 currentPiece$BFb$N%V%m%C%/$N:G>eIt$N0LCV$rJV$9(B
         +-+-+-+
        1| | | |
         +-+-+-+
        2| | | |
         +-+-+-+
        3| | | |
         +-+-+-+
******************************************************************************/
int GetPieceTop(void)
{
    int x,y,length = 2;

    for (x=0;x<3; x++) {
        if (currentPiece[x][0]) {
            length--;
            break;
        }
    }
    return length;
}

/******************************************************************************
 currentPiece$BFb$N%V%m%C%/$N:G2<It$N0LCV$rJV$9(B
         +-+-+-+
        1| | | |
         +-+-+-+
        2| | | |
         +-+-+-+
        3| | | |
         +-+-+-+
******************************************************************************/
int GetPieceBottom(void)
{
    int x,y,length = 2;

    for (x=0; x<3; x++) {
        if (currentPiece[x][2]) {
            length++;
            break;
        }
    }
    return length;
}

/******************************************************************************
 currentPiece$BFb$N%V%m%C%/$N:8B&$N0LCV$rJV$9(B
          1 2 3
         +-+-+-+
         | | | |
         +-+-+-+
         | | | |
         +-+-+-+
         | | | |
         +-+-+-+
******************************************************************************/
int GetPieceLeft(void)
{
    int x,y,length = 2;

    for (y=0; y<3; y++) {
        if (currentPiece[0][y]) {
            length--;
            break;
        }
    }
    return length;
}

/******************************************************************************
 currentPiece$BFb$N%V%m%C%/$N1&B&$N0LCV$rJV$9(B
          1 2 3
         +-+-+-+
         | | | |
         +-+-+-+
         | | | |
         +-+-+-+
         | | | |
         +-+-+-+
******************************************************************************/
int GetPieceRight(void)
{
    int x,y,length = 2;

    for (y=0; y<3; y++) {
        if (currentPiece[2][y]) {
            length++;
            break;
        }
    }
    return length;
}

/******************************************************************************
 $B%V%m%C%/$r0LCV>pJs$K=>$C$F%U%#!<%k%I$K%3%T!<$9$k(B
******************************************************************************/
void PieceToField(void)
{
    int x,y;

    for (y=0; y<3 ; y++){
        for (x=0; x<3; x++){
            if (currentPiece[x][y]){
                if (((pieceLocation_x+x)>=0)
                 && ((pieceLocation_x+x)<FIELD_WIDTH)
                 && ((pieceLocation_y+y)>=0)
                 && ((pieceLocation_y+y)<FIELD_HEIGHT)){
                    field[pieceLocation_x + x][pieceLocation_y + y] = currentPiece[x][y];
                }
            }
        }
    }
}

/******************************************************************************
 $B%V%m%C%/$N0\F0H=Dj(B
******************************************************************************/
int MovePiece(int move)
{
    int left,right,bottom;
    int x,y,count;

    // $B:80\F0(B -----------------------------------------------------------------
    if (move==PIECE_LEFT) {
        left = GetPieceLeft();  //$B%V%m%C%/:8B&$N0LCV>pJs(B

        if (pieceLocation_x > -(left-1)) {  //$B:8B&$K0\F0$G$-$k(B
            //$B0\F0@h$N%V%m%C%/$H=E$J$k$+(B?
            count = 0;
            for (y=0; y<3; y++) {
                for (x=0; x<3; x++) {
                    if (((pieceLocation_x+x-1)>=0)&& ((pieceLocation_y+y)>=0)){
                        if (currentPiece[x][y] && field[pieceLocation_x + x - 1][pieceLocation_y + y]) {
                            count++;
                        }
                    }
                }
            }
            if (!count) {
                pieceLocation_x--;  //$B0\F0(B
            }
            else {
                return FALSE;
            }
        }
        else {
            return FALSE;
        }
    }
    // $B1&0\F0(B -----------------------------------------------------------------
    else if (move==PIECE_RIGHT) {
        right = GetPieceRight();    //$B%V%m%C%/1&B&$N0LCV>pJs(B

        if (pieceLocation_x < (FIELD_WIDTH-right)) {    //$B1&B&$K0\F0$G$-$k(B
            //$B0\F0@h$N%V%m%C%/$H=E$J$k$+(B?
            count = 0;
            for (y=0; y<3; y++) {
                for (x=0; x<3; x++) {
                    if (((pieceLocation_x+x+1)>=0)&& ((pieceLocation_y+y)>=0)){
                        if (currentPiece[x][y] && field[pieceLocation_x + x + 1][pieceLocation_y + y]) {
                            count++;
                        }
                    }
                }
            }
            if (!count) {
                pieceLocation_x++;  //$B0\F0(B
            }
            else {
                return FALSE;
            }
        }
        else {
            return FALSE;
        }
    }
    // $B2<0\F0(B -----------------------------------------------------------------
    else if (move==PIECE_DOWN) {
        bottom = GetPieceBottom();  //$B%V%m%C%/2<B&$N0LCV>pJs(B

        if (pieceLocation_y < (FIELD_HEIGHT-bottom)) {  //$B2<B&$K0\F0$G$-$k(B
            //$B0\F0@h$N%V%m%C%/$H=E$J$k$+(B?
            count = 0;
            for (y=0; y<3 ; y++) {
                for (x=0; x<3; x++) {
                    if (((pieceLocation_x+x)>=0)&& ((pieceLocation_y+y+1)>=0)){
                        if (currentPiece[x][y] && field[pieceLocation_x + x][pieceLocation_y + y + 1]) {
                            count++;
                        }
                    }
                }
            }
            if (!count) {
                pieceLocation_y++;  //$B0\F0(B
            }
            else {
                return FALSE;
            }
        }
        else {
            return FALSE;
        }
    }

    return TRUE;
}

/******************************************************************************
 $B%V%m%C%/$r2sE>$5$;$k(B
******************************************************************************/
int TurnPiece(void)
{
    int x,y,offsetX;
    int copy[3][3];

    //$B2sE>$7$?%V%m%C%/$r@8@.$9$k(B
    for (y=0; y<3; y++){
        for (x=0; x<3; x++){
            copy[2-y][x] = currentPiece[x][y];
        }
    }
    //$B2sE>2DG=$+$I$&$+$rD4$Y$k(B
    for (y=0; y<3; y++){
        for (x=0; x<3; x++){
            if (copy[x][y]){
                offsetX = pieceLocation_x + x;
                if ((offsetX < 0) || (offsetX >= FIELD_WIDTH) || field[offsetX][pieceLocation_y + y])
                    return FALSE;
            }
        }
    }

    //copy$B$r(BCurrentPiece$B$K%3%T!<$9$k(B
    for (y=0; y<3; y++){
        for (x=0; x<3; x++){
            currentPiece[x][y] = copy[x][y];
        }
    }
    return TRUE;
}

/******************************************************************************
 $B3F9T$rD4$Y!$9T$,Kd$^$C$F$$$k>l9g$O9T$r:o=|$9$k(B
******************************************************************************/
int AdjustLine(void)
{
    int x,y,delCount = 0;
    int iy,lineCount;

    for (y=FIELD_HEIGHT-1; y>=0; y--){
        lineCount = 0;
        for (x=0; x<FIELD_WIDTH; x++){
            if (field[x][y]) lineCount++;   //$B$=$N9T$N%V%m%C%/$N?t$r?t$($k(B
        }

        if (lineCount==0){  //$B$3$l$h$j>e$K%V%m%C%/$O$J$$(B
            break;
        }
        else if (lineCount==FIELD_WIDTH){   //1$B9T:o=|$9$k(B
            delCount++;
            for (iy=y; iy>=0; iy--){
                for (x=0; x<FIELD_WIDTH; x++){
                    if (iy>0)   {field[x][iy] = field[x][iy-1];}
                    else        {field[x][0] = 0;}  //0$B9T$h$j>e$O$J$$$N$G(B0$B$GKd$a$k(B
                }
            }
            y++;    //$B$b$&0lEYF1$89T$+$i%A%'%C%/$9$k(B
        }
    }

    return delCount;    //$B2?9T:o=|$7$?$+(B?
}

/******************************************************************************
 $B<!$N%V%m%C%/$X(B
******************************************************************************/
void NextPiece(void)
{
    int x,y;

    for (y=0; y<3; y++){
        for (x=0; x<3; x++){
            currentPiece[x][y] = 0;
        }
    }

    //$B%V%m%C%/$NA*Br(B
    switch(ransuu0_6){
        case 0:
            currentPiece[1][0] = 1;
            currentPiece[1][1] = 1;
            currentPiece[1][2] = 1;
            break;
        case 1:
            currentPiece[1][1] = 1;
            currentPiece[2][1] = 1;
            currentPiece[1][2] = 1;
            break;
        case 2:
            currentPiece[1][1] = 1;
            currentPiece[2][1] = 1;
            currentPiece[2][2] = 1;
            break;
        case 3:
            currentPiece[0][2] = 1;
            currentPiece[1][1] = 1;
            currentPiece[1][2] = 1;
            currentPiece[2][1] = 1;
            break;
        case 4:
            currentPiece[0][1] = 1;
            currentPiece[1][1] = 1;
            currentPiece[1][2] = 1;
            currentPiece[2][2] = 1;
            break;
        case 5:
            currentPiece[0][2] = 1;
            currentPiece[1][1] = 1;
            currentPiece[1][2] = 1;
            currentPiece[2][2] = 1;
            break;
        case 6:
            currentPiece[1][1] = 1;
            currentPiece[1][2] = 1;
            currentPiece[2][1] = 1;
            currentPiece[2][2] = 1;
            break;
    }

    //$B0LCV$N=i4|@_Dj(B
    pieceLocation_x =  2;
    pieceLocation_y = -2;
}

/******************************************************************************
 $BI=<(%G!<%?%;%C%H(B
******************************************************************************/
void Paint(void)
{
    int x,y,n;
    char d,copy_field[8][8];

    //field$B$r%3%T!<(B
    for (y=0; y<8 ; y++){
        for (x=0; x<8; x++){
            copy_field[x][y] = field[x][y];
        }
    }

    //currentPiece$B$r=E$M$k(B
    for (y=0; y<3 ; y++){
        for (x=0; x<3; x++){
            if (currentPiece[x][y]){
                if (((pieceLocation_x+x)>=0)
                 && ((pieceLocation_x+x)<FIELD_WIDTH)
                 && ((pieceLocation_y+y)>=0)
                 && ((pieceLocation_y+y)<FIELD_HEIGHT)){
                    copy_field[pieceLocation_x + x][pieceLocation_y + y] = currentPiece[x][y];
                }
            }
        }
    }

    //$BI=<(%G!<%?$KJQ49(B
    for (x=0; x<8; x++){
        d = 0;
        n = 1;
        for (y=0; y<8; y++){
            d = d + (unsigned char)(copy_field[x][y] * n);
            n = n * 2;
        }
        DispBuf[x] = d;
    }
}

/******************************************************************************
 1$BIC$4$H$N%V%m%C%/H=Dj(B
******************************************************************************/
void TimerProc(void)
{
    int line, top;

    if (MovePiece(PIECE_DOWN)==FALSE){  //$B$3$l$h$j2<$K0\F0$G$-$J$$(B
        PieceToField();
        line = AdjustLine();    //$B9T%A%'%C%/!$(B1$B9TKd$^$C$F$$$?$i$=$N9T$O:o=|(B
        Point = Point + line * line;    //$B:o=|$7$?9T?t$N(B2$B>h$,E@?t$K$J$k(B
        top = GetPieceTop();
        if ((pieceLocation_y + line + top - 1) < 0){
            GameStage = 2;  //$B%2!<%`%*!<%P!<(B
        }
        else{
            NextPiece();    //$B<!$N%V%m%C%/(B
        }
    }
}

/******************************************************************************
 $B%F%H%j%9$N=i4|@_Dj(B
******************************************************************************/
void init_tetris(void)
{
    int i,j;

    for (i=0; i<FIELD_WIDTH; i++){
        for (j=0; j<FIELD_HEIGHT; j++){
            field[i][j] = 0;
        }
    }

    Point = 0;
    MoveTimCount = MOVE_TIM_1;
}
